const std = @import("std");

const Scanner = @import("zig-wayland").Scanner;

pub fn build(b: *std.Build) void {
    const target = b.standardTargetOptions(.{});
    const optimize = b.standardOptimizeOption(.{});

    const pie = b.option(bool, "pie", "Build a Position Independent Executable") orelse false;
    const zsh_completion = b.option(
        bool,
        "zsh-completion",
        "Set to true to install zsh completion for river-bedload. Defaults to true.",
    ) orelse true;

    const options = b.addOptions();

    const scanner = Scanner.create(b, .{});

    scanner.addCustomProtocol("protocol/river-status-unstable-v1.xml");

    scanner.generate("wl_compositor", 4);
    scanner.generate("wl_shm", 1);
    scanner.generate("wl_output", 4);
    scanner.generate("wl_seat", 5);
    scanner.generate("zriver_status_manager_v1", 4);

    const exe = b.addExecutable(.{
        .name = "river-bedload",
        .root_source_file = b.path("src/main.zig"),
        .target = target,
        .optimize = optimize,
    });
    exe.root_module.addOptions("build_options", options);

    const wayland = b.createModule(.{ .root_source_file = scanner.result });
    const flags = b.createModule(.{ .root_source_file = b.path("common/flags.zig") });

    exe.linkLibC();

    exe.root_module.addImport("wayland", wayland);
    exe.linkSystemLibrary("wayland-client");

    exe.root_module.addImport("flags", flags);

    // TODO: remove when https://github.com/ziglang/zig/issues/131 is implemented
    scanner.addCSource(exe);

    exe.pie = pie;

    b.installArtifact(exe);
    b.installFile("doc/river-bedload.1", "share/man/man1/river-bedload.1");

    if (zsh_completion) {
        b.installFile("completion/zsh/_river_bedload", "share/zsh/site-functions/_river_bedload");
    }
}
