// This file is part of river-bedload
//
// Copyright (C) 2023 Hugo Machet
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 3
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

const std = @import("std");
const mem = std.mem;
const posix = std.posix;

const wayland = @import("wayland");
const wl = wayland.client.wl;

const ctx = &@import("main.zig").context;
const log = std.log.scoped(.event_loop);

pub fn run() !void {
    const poll_wayland = 0;

    var pfds: [1]posix.pollfd = undefined;

    pfds[poll_wayland] = .{
        .fd = ctx.backend.display.getFd(),
        .events = posix.POLL.IN,
        .revents = 0,
    };

    var bw = std.io.bufferedWriter(std.io.getStdOut().writer());
    const w = bw.writer();

    while (ctx.initialized) {
        ctx.backend.flush_wayland_and_prepare_read();

        try ctx.backend.print_json(w);
        try bw.flush();

        _ = posix.poll(&pfds, -1) catch |err| {
            ctx.fatal(.event_loop, "failed to poll {s}", .{@errorName(err)});
        };

        if (pfds[poll_wayland].revents & posix.POLL.IN != 0) {
            const errno = ctx.backend.display.readEvents();
            if (errno != .SUCCESS) {
                ctx.fatal(.event_loop, "error reading wayland events: {s}", .{@tagName(errno)});
            }
        } else {
            ctx.backend.display.cancelRead();
        }
    }

    ctx.backend.flush_wayland_and_prepare_read();
}
